import 'dart:async';

enum PasswordStrength {
  tiny,
  veryWeek,
  week,
  good,
  strong,
}

class PasswordStrengthVerificator {
  final int minPasswordLength;

  const PasswordStrengthVerificator(this.minPasswordLength);

  FutureOr<PasswordStrength> call(FutureOr<String> password) async {
    PasswordStrength response = PasswordStrength.veryWeek;
    final realPassword = (await password).trim();
    if (realPassword.length < minPasswordLength) {
      response = PasswordStrength.tiny;
    } else {
      int points = 0;
      if (lowerCase.hasMatch(realPassword)) {
        points += 1;
      }

      if (upperCase.hasMatch(realPassword)) {
        points += 1;
      }

      if (numberCase.hasMatch(realPassword)) {
        points += 1;
      }

      if (symbolsCase.hasMatch(realPassword)) {
        points += 1;
      }

      switch (points) {
        case 0:
          response = PasswordStrength.veryWeek;
          break;
        case 1:
          response = PasswordStrength.veryWeek;
          break;
        case 2:
          response = PasswordStrength.week;
          break;
        case 3:
          response = PasswordStrength.good;
          break;
        case 4:
          response = PasswordStrength.strong;
          break;
      
      }
    }

    return response;
  }

  RegExp get lowerCase => RegExp(
        r'^(?=.*?[a-z]).{'
        '$minPasswordLength'
        r',}$',
      );

  RegExp get upperCase => RegExp(
        r'^(?=.*?[A-Z]).{'
        '$minPasswordLength'
        r',}$',
      );

  RegExp get numberCase => RegExp(
        r'^(?=.*?[0-9]).{'
        '$minPasswordLength'
        r',}$',
      );

  RegExp get symbolsCase => RegExp(
        r'^(?=.*?[!@#\$&*~]).{'
        '$minPasswordLength'
        r',}$',
      );

  RegExp get strongPasswordRegExp => RegExp(
        r'^(?=.*?[A-Z])(?=.*?[a-z])(?=.*?[0-9])(?=.*?[!@#\$&*~]).{'
        '$minPasswordLength'
        r',}$',
      );

  RegExp get goodPasswordRegExp => RegExp(
        r'^(?=.*?[A-Z])(?=.*?[a-z])(?=.*?[0-9]).{'
        '$minPasswordLength'
        r',}$',
      );

  RegExp get weekPasswordRegExp => RegExp(
        r'^(?=.*?[A-Z])(?=.*?[a-z]).{'
        '$minPasswordLength'
        r',}$',
      );
}
